@@ -1,7 +1,12 @@
 """MQTT Vacuum entity module."""
+import asyncio
 import json
 import logging
-from typing import Any, Dict, Optional, Union
+from collections.abc import Callable
+from contextlib import AsyncExitStack
+from typing import Any
+
+import asyncio_mqtt as mqtt
 
 from mqtt_hass_base.entity.common import EntitySettingsType, MqttEntity
 from mqtt_hass_base.error import MQTTHassBaseError
@@ -17,11 +22,13 @@ VACUUM_STATES = (
 )
 
 
-class VacuumSettingsType(EntitySettingsType, total=False):
+class VacuumSettingsType(
+    EntitySettingsType, total=False
+):  # pylint: disable=too-few-public-methods
     """Vacuum entity settings dict format."""
 
     supported_features: list[str]
-    fan_speed_list: Optional[Union[str, list[str]]]
+    fan_speed_list: str | list[str] | None
 
 
 class MqttVacuum(MqttEntity):
@@ -45,33 +52,37 @@ class MqttVacuum(MqttEntity):
         self,
         name: str,
         unique_id: str,
+        mqtt_client: mqtt.Client,
         mqtt_discovery_root_topic: str,
         mqtt_data_root_topic: str,
         logger: logging.Logger,
-        device_payload: Dict[str, str],
-        subscriptions: Dict[str, str],
+        device_payload: dict[str, str],
+        subscriptions: dict[str, Callable[..., Any]],
         supported_features: list[str],
-        fan_speed_list: Optional[Union[str, list[str]]],
-        object_id: Optional[str] = None,
+        fan_speed_list: str | list[str] | None,
+        object_id: str | None = None,
+        entity_category: str | None = None,
     ):
         """Create a new Vacuum entity."""
         MqttEntity.__init__(
             self,
             name,
             unique_id,
+            mqtt_client,
             mqtt_discovery_root_topic,
             mqtt_data_root_topic,
             logger,
             device_payload,
             subscriptions,
             object_id=object_id,
+            entity_category=entity_category,
         )
         self._supported_features = supported_features
         self._fan_speed_list = fan_speed_list
-        self._state: Union[str, bytes, float, int, None] = None
+        self._state: str | bytes | float | int | None = None
 
     @property
-    def state(self) -> Union[str, bytes, float, int, None]:
+    def state(self) -> str | bytes | float | int | None:
         """Get the current state of the vacuum."""
         return self._state
 
@@ -89,7 +100,7 @@ class MqttVacuum(MqttEntity):
             (self.mqtt_data_root_topic, self._component, self.name, "set_fan_speed")
         ).lower()
 
-    def register(self) -> None:
+    async def register(self) -> None:
         """Register the current entity to Hass.
 
         Using the MQTT discovery feature of Home Assistant.
@@ -123,32 +134,76 @@ class MqttVacuum(MqttEntity):
         }
 
         if self._object_id:
-            config_payload["config_object_id"] = self._object_id
+            config_payload["config_object_id"] = self.object_id
+        if self.entity_category:
+            config_payload["entity_category"] = self.entity_category
 
         self.logger.debug("%s: %s", self.config_topic, json.dumps(config_payload))
-        self._mqtt_client.publish(
+        await self._mqtt_client.publish(
             topic=self.config_topic, retain=True, payload=json.dumps(config_payload)
         )
 
-    def send_state(
+    async def subscribe(self, stack: AsyncExitStack) -> set[asyncio.Task[None]]:
+        """Subscribe to all mqtt topics needed."""
+        tasks: set[asyncio.Task[None]] = set()
+        if on_command_callback := self._subscriptions.get("command_topic"):
+            messages = await stack.enter_async_context(
+                self._mqtt_client.filtered_messages(self.command_topic)
+            )
+            tasks.add(
+                asyncio.create_task(self._on_messages(messages, on_command_callback))
+            )
+            # Subscribe to topic(s)
+            # 🤔 Note that we subscribe *after* starting the message
+            # loggers. Otherwise, we may miss retained messages.
+            await self._mqtt_client.subscribe(self.command_topic)
+
+        if on_send_command_callback := self._subscriptions.get("send_command_topic"):
+            messages = await stack.enter_async_context(
+                self._mqtt_client.filtered_messages(self.send_command_topic)
+            )
+            tasks.add(
+                asyncio.create_task(
+                    self._on_messages(messages, on_send_command_callback)
+                )
+            )
+            # Subscribe to topic(s)
+            # 🤔 Note that we subscribe *after* starting the message
+            # loggers. Otherwise, we may miss retained messages.
+            await self._mqtt_client.subscribe(self.send_command_topic)
+
+        if set_fan_speed_callback := self._subscriptions.get("set_fan_speed_topic"):
+            messages = await stack.enter_async_context(
+                self._mqtt_client.filtered_messages(self.set_fan_speed_topic)
+            )
+            tasks.add(
+                asyncio.create_task(self._on_messages(messages, set_fan_speed_callback))
+            )
+            # Subscribe to topic(s)
+            # 🤔 Note that we subscribe *after* starting the message
+            # loggers. Otherwise, we may miss retained messages.
+            await self._mqtt_client.subscribe(self.set_fan_speed_topic)
+
+        return tasks
+
+    async def send_state(
         self,
-        state: Union[str, bytes, float, int],
-        attributes: Optional[Dict[str, Any]] = None,
-        battery_level: Optional[float] = None,
-        fan_speed: Optional[str] = None,
+        state: str | bytes | float | int,
+        attributes: dict[str, Any] | None = None,
+        battery_level: float | None = None,
+        fan_speed: str | None = None,
     ) -> None:  # pylint: disable=arguments-differ
         """Send the current state of the vacuum to Hass."""
         if state not in VACUUM_STATES:
             raise MQTTHassBaseError("Bad state {state} not in {VACUUM_STATES}")
-        payload: Dict[str, Any] = {}
+        payload: dict[str, Any] = {}
         payload["state"] = state
         self._state = state
         if battery_level:
             payload["battery_level"] = battery_level
         if fan_speed:
             payload["fan_speed"] = fan_speed
-        self._mqtt_client.publish(
+        await self._mqtt_client.publish(
             topic=self.state_topic, retain=True, payload=json.dumps(payload)
         )
-        if attributes is not None:
-            self.send_attributes(attributes)
+        await self.send_attributes(attributes)
